#include <Adafruit_GFX.h>    // Core graphics library
#include <MCUFRIEND_kbv.h>   // Hardware-specific library
#include "AffichageTS35.h"  
MCUFRIEND_kbv tft;

// =================== Touch Screen constantes =================== //
// To change according to the calibration
// Valeurs à remplacer par celle données lors de la calibration
int XP=8,XM=A2,YP=A3,YM=9; //320x480 ID=0x9486
const int TS_LEFT=125,TS_RT=900,TS_TOP=942,TS_BOT=95;


#include "TouchScreen_kbv.h"         //hacked version
TouchScreen_kbv ts(XP, YP, XM, YM, 300);   //re-initialised after diagnose
TSPoint_kbv tp;                            //global point




// Fontes utilisées par le programme
#include <Fonts/FreeMonoBold12pt7b.h>
#include <Fonts/FreeSans12pt7b.h>
#include <Fonts/FreeMono9pt7b.h>
#include <FreeDefaultFonts.h>

// constante pour l'écran LARGEUR_ECRANxHAUTEUR_ECRAN -> affichage mesure

#define HautBouton 25

#define LigneTexte 25        // 1ere Ligne texte
#define CouleurTexte BLANC  // Couleur du texte 
#define CouleurValeur VERT  //

#define HauteurBoutonClavier 25
#define LargeurBoutonClavier 80
#define HauteurGrandBouton 25	// Hauteur d'un grand bouton par défaut
#define COLCLAV (LARGEUR_ECRAN-(3*LargeurBoutonClavier+20))/2

TBouton Clavier[14] = {{60,COLCLAV,LargeurBoutonClavier,"1",BLANC,CouleurFond,ROUGE,BLANC},
      {60,COLCLAV+10+LargeurBoutonClavier,LargeurBoutonClavier,"2",BLANC,CouleurFond,ROUGE,BLANC},
      {60,COLCLAV+20+2*LargeurBoutonClavier,LargeurBoutonClavier,"3",BLANC,CouleurFond,ROUGE,BLANC},
      {90,COLCLAV,LargeurBoutonClavier,"4",BLANC,CouleurFond,ROUGE,BLANC},
      {90,COLCLAV+10+LargeurBoutonClavier,LargeurBoutonClavier,"5",BLANC,CouleurFond,ROUGE,BLANC},
      {90,COLCLAV+20+2*LargeurBoutonClavier,LargeurBoutonClavier,"6",BLANC,CouleurFond,ROUGE,BLANC},
      {120,COLCLAV,LargeurBoutonClavier,"7",BLANC,CouleurFond,ROUGE,BLANC},
      {120,COLCLAV+10+LargeurBoutonClavier,LargeurBoutonClavier,"8",BLANC,CouleurFond,ROUGE,BLANC},
      {120,COLCLAV+20+2*LargeurBoutonClavier,LargeurBoutonClavier,"9",BLANC,CouleurFond,ROUGE,BLANC},
      {150,COLCLAV+10+LargeurBoutonClavier,LargeurBoutonClavier,"0",BLANC,CouleurFond,ROUGE,BLANC},
      {190,COLCLAV,LargeurBoutonClavier,"V",VERT,CouleurFond,ROUGE,BLANC},
      {190,COLCLAV+10+LargeurBoutonClavier,LargeurBoutonClavier,"C",JAUNE,CouleurFond,ROUGE,BLANC},
      {190,COLCLAV+20+2*LargeurBoutonClavier,LargeurBoutonClavier,"A",ROUGE,BLANC,ROUGE,BLANC},
      {-1,-1,0,"",0,0,0,0}};
TBouton Afficheur[2] = {{20,(LARGEUR_ECRAN-2*LargeurCol)/2,2*LargeurCol,"      ",BLANC,NOIR,BLANC, ROUGE},
            {-1,-1,0,"",0,0,0,0}};


/*
 * Init affichage écran
 */
void AfficheInit()
{
    uint16_t ID = tft.readID();
    if (ID == 0xD3) ID = 0x9481;
    tft.begin(ID);
    tft.setRotation(1);  
}

/*
 * Affiche un texte centré sur l'écran
 * ligne : ligne en pixel 
 * Couleur : couleur du texte
 * msg : texte à afficher
 */
void AfficheCentre(int ligne, const unsigned int Couleur, const char *msg) 
{
  int x1, y1, w, h, decal;
  tft.getTextBounds(msg, 0, 0,&x1, &y1, &w, &h);
  decal = (LARGEUR_ECRAN-w)/2;
  if (decal < 0) {decal = 0;}
  tft.setCursor(decal,ligne+h+10);
  tft.setTextColor(Couleur);
  tft.print(msg); 
}


/*
 * Affiche l'écran d'accueil
 */
void AfficheEcranAccueil()
{
  tft.setFont(&FreeMonoBold12pt7b);
  AfficheCentre(10,BLANC,"CHRONOCOMPARATEUR PC-RM5");
  tft.setFont(&FreeMono9pt7b);
  AfficheCentre(100,BLANC,"Version 1.1");
  AfficheCentre(130,BLANC,"https://www.pascalchour.fr");
}


/*
 * affiche l'écran de mesure
 * PBouton : L'appelant doit fournir un pointeur sur un tableau de structure de boutons valide (au moins un initialisée à -1, -1)
 */
void AfficheEcranMenu(const TBouton PBouton1[])
{
  tft.fillScreen(CouleurFond);
  tft.setFont(&FreeMonoBold12pt7b);
  tft.setTextColor(CouleurTexte);
  tft.setCursor(5,LigneTexte);
  tft.print("P. Moy");
  tft.setCursor(5,LigneTexte+40);
  tft.print("P. Moy10");
  tft.setCursor(5,LigneTexte+80);
  tft.print("Diff Batt");
  tft.setCursor(5,LigneTexte+120);
  tft.print("A/R jour");
  tft.setCursor(5,LigneTexte+160);
  tft.print("P. Visee"); 
  tft.drawFastHLine(0,LigneBouton-5,tft.width()-1,TFT_WHITE);
  AfficheBoutons(PBouton1);
}

/*
 * Formatte un uint16 en ASCII. Le buffer doit faire 7 caractères au minimum 
 * En entrée :  Valeur = Valeur à convertir
 *
 * En sortie : 	Buffer = Resultat (7 caractères minimum)
 * 				Pas de contrôle de taille
 */

void ULongAscii(unsigned long Valeur, char Buffer[])
{
    int16_t j = 0;
    if (Valeur == 0) {
    	Buffer[0] = '0';
    	Buffer[1] = '\x0';
    } else {
    	Buffer[6] = '\x0';
      j = 5;
      while ((Valeur > 0) && (j >= 0))  {
        Buffer[j] = Valeur - ((Valeur/10)*10) + 0x30;
        Valeur = Valeur/10;
        j = j-1;
      }
      memcpy(Buffer,&Buffer[j+1],6-j);
    }
}



/*
 * affiche un clavier de saisie de nombres entiers compris entre 1 et 99999
 * En entrée : Valeur = Valeur par défaut et résultat
 * En sortie : true si saisie validée, faux sinon, saisie dans Valeur si true, sinon,inchangée
 */
bool SaisieClavier(unsigned long *Valeur)
{
  int i;
  unsigned long V = *Valeur;
  bool Modif = false;
  tft.fillScreen(CouleurFond);
  AfficheBoutons(Clavier);
  V = *Valeur;
  ULongAscii(V,Afficheur[0].buf);
  AfficheUnBouton(&Afficheur[0], false);
  while (1) {
	  if (IsPressed()) { // Appui sur le TouchScreen ?
		  i = NumBouton(Clavier);
		  if (i != -1) {
			  ClignoteBouton(&Clavier[i]);
			  if ((i >= 0) && (i <= 9) && ((V <= 9999) || (!Modif))) {
				  if (!Modif) V = 0;
				  V = V*10+((i+1)%10);
				  Modif = true;
			  } else {
				  switch (i) {
				  case 10: // Validation saisie
						  if ((V > 9999) || (!Modif))  return false;
						  *Valeur = V;
						  return true;
					  break;
				  case 11: // Correction (Clear)
						  V = 0;
						  Modif = true;
					  break;
				  case 12: //Annulation
						  return false;
					  break;
				  }
			  }
        ULongAscii(V,Afficheur[0].buf);
			  AfficheUnBouton(&Afficheur[0], false);
		  } 
	  }
  }
}




/*
 * Affiche un texte à l'endroit désigné par "bloc"
 * Efface le bloc au préalable (rempli un rectangle avec la couleur de fond couleurFondd) 
 * Les coordonnées du bloc sont déduite de la structure TextesFixes[]
 * Bloc = 1 : A/R Jour
 * Bloc = 2 : Diff. Batt
 * Bloc = 3 : P Visé
 * Bloc = 4 : P. Moy
 * Bloc = 5 : P. Moy10
  */
void AfficheBloc(int bloc, unsigned int couleurFond, unsigned int couleurCar,  const char *msg)
{
  int col,lgn;
  int i = 0;
  int j = 0;
  int L = 20;
// Calculer largeur texte à afficher (à faire) 
  if (bloc < 5) {
    tft.setFont(&FreeMonoBold12pt7b);
    tft.setTextColor(couleurCar);
    lgn = LigneTexte+40*bloc;
    tft.fillRect(150,lgn-14, LARGEUR_ECRAN-150, 20, CouleurFond);
    while ((msg[i] != '\0') && (i < MaxCarBuf)) i++;
    col =150+(MaxCarBuf-i)*14;
    tft.setCursor(col,lgn);
    tft.print(msg);    
  }
}

/*
 * Affiche les boutons de la structure passée en paramètre
 */

void AfficheBoutons(const TBouton PBouton[])
{
  int i;
  i = 0;
  while (PBouton[i].ligne != -1) {
    AfficheUnBouton(&PBouton[i],false);
    i = i+1;
  }
}


/*
 * Affiche un bouton dont les coordonnées sont données en paramètre = PBouton
 * Affiche le texte centré dans le bouton si c'est possible
 * Selection = vrai : bouton sélectionné, = faux, bouton normal
 */
void AfficheUnBouton(const TBouton *PBouton,  bool Selection)
{
  int x1, y1, w, h, decal,i;
  unsigned int ColFond, ColCar;

  if (Selection) {
    ColFond = PBouton->couleurFondSel;
    ColCar = PBouton->couleurCarSel;
  } else {
    ColFond = PBouton->couleurFond;
    ColCar = PBouton->couleurCar;
  }
  tft.setFont(&FreeSans12pt7b);
  i = 0;
  tft.getTextBounds(PBouton->buf, 0, 0,&x1, &y1, &w, &h);
  decal = PBouton->largeur-w;
  if (decal < 0) {decal = 0;}
  decal = decal / 2;
  tft.fillRoundRect(PBouton->colonne, PBouton->ligne, PBouton->largeur,HautBouton,3,ColFond);
  tft.setCursor(PBouton->colonne+decal,PBouton->ligne+HautBouton-5);
  tft.setTextColor(ColCar);
  tft.print(PBouton->buf);
}

/*
 * Affiche le bouton passé en paramètre et fait basculer les couleurs entre bouton sélectionné et bouton non sélectionné 
 * pour donner une impression de clignotement
 */
void ClignoteBouton(const TBouton *PBouton)
{
  int i;
  for (i=0; i<5; i++) {
    AfficheUnBouton(PBouton, true);
    delay(20);
    AfficheUnBouton(PBouton, false);
    delay(20);
  }
}


/*
 * Donne l'index du n° du bouton pressé dans le tableau de structure donné en paramètre
 * Les coordonnées du dernier bouton doivent valoir -1 pour arrêter la recherche dans le tableau
 * La fonction renvoie -1 si aucune coordonnées de bouton correspond aux coordonnées où il y a eu appuie sur l'écran
 */

int NumBouton(const TBouton PBouton[])
{
  long x;
  long y;
  int i = 0;
// Attention : les coordonnées de l'écran dépendent de son orientation
// Dans le cas présent, les coordonnées "lignes" sont inversées d'où HAUTEUR_ECRAN, 0 au lieu de 0, HAUTEUR_ECRAN. 
// Faire des essais si problème et afficher les coordonnées en retirant le commentaire
// sur la ligne serial.println
  x = map(tp.x, TS_LEFT, TS_RT, HAUTEUR_ECRAN, 0);
  y = map(tp.y, TS_TOP, TS_BOT, 0, LARGEUR_ECRAN);
// Serial.println("x=" + String(x) + ", y=" + String(y) + ", tp.z =" + String(tp.z));  
  while (PBouton[i].ligne != -1) {
    if ((x >= PBouton[i].ligne) && (x < PBouton[i].ligne+HautBouton)) {
      if ((y >= PBouton[i].colonne) && (y <= PBouton[i].colonne+PBouton[i].largeur)) {
        return i;
      }
    }
    i = i+1;
  }
  return -1;
}

/*
 * Lit les valeurs d'un appui sur l'écran (X, Y, Z)
 */
void readResistiveTouch(void)
{
    tp = ts.getPoint();
    pinMode(YP, OUTPUT);      //restore shared pins
    pinMode(XM, OUTPUT);
    digitalWrite(YP, HIGH);  //because TFT control pins
    digitalWrite(XM, HIGH);
    //Serial.println("tp.x=" + String(tp.x) + ", tp.y=" + String(tp.y) + ", tp.z =" + String(tp.z));
}

/*
 * Indique si on a appuyé sur l'écran
 */
 
bool IsPressed(void)
{
    // .kbv this was too sensitive !!
    // now touch has to be stable for 50ms
    int count = 0;
    bool state, oldstate;
    while (count < 10) {
        readResistiveTouch();
        state = tp.z > 200;     //ADJUST THIS VALUE TO SUIT YOUR SCREEN e.g. 20 ... 250
        if (state == oldstate) count++;
        else count = 0;
        oldstate = state;
        delay(5);
    }
    return oldstate;
}


